package minicollection;

/**
 * A basic collection type for elements of type E.
 * The basic collection only has operations which do not alter the membership
 * of the collection (add elements, remove elements, or change elements through
 * an operation of the collection).
 * 
 * This does not guarantee the immutability of the collection.  If the elements 
 * themselves are mutable, any of those elements themselves can be changed by 
 * obtaining a reference to it.  Such a reference could be obtained, for example,
 * from the IMiniIterator since this type extends IMiniIterable, but the 
 * existence of other references to collection members is out of the control 
 * of the collection. 
 * 
 * IMiniBasicCollection extends Iterable so that the for-each construct can be used
 * with IMiniBasicCollection types.    In an ideal world, when MiniCollections take 
 * over from the Java Collection Framework, or the Java people are willing 
 * to extend for-each construct to use IMiniIterators, IMiniBasicCollection would 
 * not need to extend Iterable.  At present, extending Iterable means that 
 * IMiniBasicCollection types must provide a Java Iterator, which has a 
 * stupid remove method which we really do not want.
 * 
 * @author jah217
 *
 */
public interface IMiniBasicCollection<E> 
			extends IMiniIterable<E>, Iterable<E> {
	
	/**
	 * Returns the membership mode of the collection.
	 * This relates to the maximum number of copies of each unique element
	 * permitted in the collection.  
	 * For example, returns ANYTHING GOES if there are no restrictions on 
	 * the cardinality of each element.  Returns UNIQUES ONLY if each unique
	 * element may only be present once in the collection.
	 * 
	 * @pre void
	 * @post void
	 * @return the membership mode of the collection.
	 */
	MembershipMode getMembershipMode();
	
	
	/**
     * Return the number of elements in the collection.
     * Returns 0 if and only if isEmpty == true.
     * 
     * @pre void
	 * @post void
	 * @return the number of elements in the collection
     */
    int size();
    
    /**
     * Checks if the collection is empty.
     * Returns true if and only if there are no elements in the collection,
     * ie when size() == 0.
     * 
     * @pre void
	 * @post void
	 * @return true if the collection contains no elements, false otherwise
     */
    boolean isEmpty();
    
    /**
     * Checks if the collection contains a specified object.
     * Returns true if and only if the collection contains at least one
     * element e for which e.equals(element) == true, ie if
     * count(element) > 0.
     * 
     * @pre void
	 * @post void
	 * @param element the object to look for in the collection
     * @return true if and only if the collection contains element
     */
    boolean contains(Object element);
    
    /**
     * Return a count of the number of copies of a specified object in the collection.
     * Returns 0 if and only if collection does not contain the object, ie 
     * if contains(element) == false.  Otherwise returns the number of elements e
     * in the collection for which e.equals(element) == true.
    
     * @pre void
	 * @post void
	 * @param element the object to count occurrences of in the collection
     * @return the number of occurrences of element in the collection
     */
    int count(Object element);
       
    
}
