package minicollection;

import java.util.Iterator;

/**
 * A type which implements a collection for elements of type E.
 * The implementation may have operations which alter the membership
 * of the collection (add elements, remove elements, or change elements through
 * an operation of the collection).
 * 
 * @author jah217
 *
 */

public interface IMiniCollectionImplementation<E> {
	
	/**
     * Return the number of elements in the collection implementation.
     * Returns 0 if and only if implIsEmpty == true.
     * 
     * @pre void
	 * @post void
	 * @return the number of elements in the collection implementation
     */
    int implSize();
    
    /**
     * Checks if the collection is empty.
     * Returns true if and only if there are no elements in the 
     * collection implementation, ie when implSize() == 0.
     * 
     * @pre void
	 * @post void
	 * @return true if the collection implementation contains no elements, false otherwise
     */
    boolean implIsEmpty();
    
    /**
     * Checks if the collection implementation contains a specified object.
     * Returns true if and only if the collection implementation contains at
     * least one element e for which e.equals(element) == true, ie if
     * implCount(element) > 0.
     * 
     * @pre void
	 * @post void
	 * @param element the object to look for in the collection implementation
     * @return true if and only if the collection implementation contains element
     */
    boolean implContains(Object element);
    
    /**
     * Return a count of the number of copies of a specified object in the
     * collection implementation.
     * Returns 0 if and only if collection implementation does not contain the object, ie 
     * if implContains(element) == false.  Otherwise returns the number of elements e
     * in the collection for which e.equals(element) == true.
    
     * @pre void
	 * @post void
	 * @param element the object to count occurrences of in the collection implementation
     * @return the number of occurrences of element in the collection implementation
     */
    int implCount(Object element); // duplicates count?
    
    /**
     * Empties the collection implementation.
     * 
     * @pre	 void  
     * @post there are no elements in the collection implementation
     * 		(implIsEmpty() == true) 
     * 
     * @return true if the operation has been completed.
     */
    boolean implClear();
    
    /**
     * Removes duplicate elements in the collection implementation.
     * Two elements e1 and e2 are duplicates if e2.equals(e2) == true.
     *
     * @pre void  
     * @post there are no two elements e1 and e2 in the collection 
     * implementation for which e2.equals(e2) == true. 
     * 
     */
    void implRemoveDuplicates();
	
    /**
	 * Return an IMiniIterator over a collection of elements of type E.
	 * 
	 * @return an IMiniIterator.
	 */
	IMiniIterator<E> implMiniIterator(); 
    
	/**
	 * Return an Iterator over a collection of elements of type E.
	 * 
	 * @return an Iterator.
	 */
	Iterator<E> implIterator(); 

    
    
}
