package minicollection;

import java.util.Comparator;

/**
 * A collection type for elements of type E where each element has an
 * ordered position or index in the collection.
 * 
 * @author jah217
 *
 */

public interface IMiniSequence<E> 
					extends IMiniCollection<E> {
	
	/**
	 * Checks the ordering mode of the sequence.
	 * The ordering mode is a description of the way in which the 
	 * elements are ordered in the collection.  
	 * For example, if elements ordered according to the order of insertion,
	 * the ordering mode will be INSERTION_ORDERING.  If elements are ordered
	 * by their natural ordering, the ordering mode will be NATURAL_ORDERING.
	 * If elements are ordered according to some other Comparator function, 
	 * the ordering mode will be CUSTOM_ORDERING.  If the ordering mode 
	 * is not purely one of these the ordering mode will be MIXED_ORDERING.
	 *  
	 * @pre void
	 * @post void
	 * @return the ordering mode of the sequence
	 */
	OrderingMode getOrderingMode();
	
	/**
	 * Checks the access mode of the sequence.
	 * The access mode is a description of the way in which the 
	 * elements can be accessed.
	 * For example, if elements can be accessed in a random order then
	 * the access mode is RANDOM_ACCESS.  If elements can only be accessed
	 * by processing each element in the sequence in order, then the 
	 * access mode is SEQUENTIAL_ACCESS.  
	 * 
	 * @pre void
	 * @post void
	 * @return the access mode of the collection. 
	 */
	AccessMode getAccessMode();
	
	/**
	 * Add an element to the the sequence.  The sequence should attempt
	 * to add the element to the elements it contains and return an enumerated 
	 * type specifying if and how the element was added to the sequence.  
	 * The operation may return FAILED if and only if the attempt to add 
	 * the element fails for reasons beyond the control of the sequence.
	 * If the addition of the element results in the replacement of any
	 * pre-existing member of the collection with the element, the 
	 * operation should return OVERWRITTEN.  If the addition of the element
	 * does not result in the replacement of a previously existing member
	 * of the collection with the element the operation should return NEW.
	 *  
	 * @pre void
	 * @post -if the operation returns NEW then the element has been added to 
	 * the the sequence and all members of the sequence prior to the inception
	 * of the operation are still in the sequence.  If the operation returns
	 * OVERWRITTEN then the element has replaced an element e in the sequence
	 * which was present at the inception of the operation and for which 
	 * e.equals(element) == true.
	 * 		- There is no change in the ordering mode of the sequence. 
	 * @param element the element to try to add to the collection. 
	 * @return an InsertCheck value reflecting the result of the add operation.
	 */
	InsertCheck add (E element);
	
	/**
	 * Remove all occurrence of a specified element o in the sequence.
	 * Removes from the sequence any element e for which e.equals(o) == true.
	 * 
	 * @pre void
	 * @post there is no element e in the sequence for which e.equals(o) == true
	 * @param o the element to remove 
	 * @return true when the operation has been successfully completed.
	 */
	boolean removeAll (E o);
	
	/**
	 * Sort the elements in the collection using their natural ordering. 
	 * This method will fail if the type E does not implement Comparable.
	 * 
	 * @pre the type E of the elements in the collection implements Comparable.
	 * @post the elements in the collection will be sorted according to
	 * their natural ordering. 
	 * 
	 */
	void sort();
	
	/**
	 * Sort the elements in the collection using the supplied Comparator.
	 * 
	 * @pre void.
	 * @post the elements in the collection will be sorted according to
	 * the ordering specified by the supplied Comparator. 
	 * @param c the Comparator with which to sort.
	 */
	void sort(Comparator<? super E> c);
    
	/**
	 * Return the element at the specified index in the sequence.
	 * @throws an IndexOutOfBoundsException if index < 0 or index >= size()
	 * @pre 0 <= index < size()
	 * @post void
	 * @param index the index value of the element to be returned.
	 * @return the element at the specified index in the sequence
	 */
	E getAtIndex(int index);
    
	/**
     * Returns the index of the first occurrence of the specified object 
     * in the sequence. Returns -1 if and only if the object is not in 
     * the sequence.
     * 
     * @pre void
	 * @post has returned -1 if and only if the object was not in the sequence,
	 * or has returned the index value i of the first occurrence in the 
	 * sequence, 0 < i < size().
	 * @param o the object for which the first index value is sought.
     * @return the value of the index of the first occurrence of the 
     * object, or -1 if the object is not in the sequence. 
     */
    int firstIndexOf(Object o);
    
    /**
     * Returns the index of the next occurrence of the specified object 
     * in the sequence at or after the specified index value.
     * Returns -1 if and only if the object is not in 
     * the sequence at an index i, index <= i < size().
     * 
	 * @throws an IndexOutOfBoundsException if index < 0 or index >= size()
	 * @pre 0 <= index < size()
	 * @post has returned -1 if and only if the object was not in the 
	 * sequence an index i, index <= i < size(), 
	 * or has returned the index value i of the first occurrence in the 
	 * sequence, i >= index.
	 * @param o the object for which the next index value is sought.
     * @param index the index value from which to start to look for
     * the next occurrence of the object.
     * @return the value of the index of the first occurrence of the 
     * object at or after index, or -1 if the object is not in the 
     * sequence at any , index <= i < size(). 
     */
    int nextIndexOf(int index, Object o );
    
    /**
     * Returns the index of the last occurrence of the specified object 
     * in the sequence. Returns -1 if and only if the object is not in 
     * the sequence.
     * 
	 * @pre void
	 * @post has returned -1 if and only if the object was not in the sequence,
	 * or has returned the index value i of the last occurrence in the 
	 * sequence, 0 < i < size().
	 * @param o the object for which the last index value is sought.
     * @return the value of the index of the last occurrence of the 
     * object, or -1 if the object is not in the sequence. 
     */
    int lastIndexOf(Object o);

    /**
     * Returns an IMiniSequenceIterator<E> iterator to the sequence 
     * pointing to the first element in the sequence, or to null 
     * if there are no elements in the sequence
     * 
	 * @pre void
	 * @post an iterator has been returned pointing to the element at 
	 * the beginning of the sequence or to null if size() == 0.
	 * @return an IMiniSequenceIterator<E> iterator to the sequence 
     * pointing to the first element in the sequence
     */
    IMiniSequenceIterator<E> begin(); 
    
    /**
     * Returns an IMiniSequenceIterator<E> iterator to the sequence 
     * pointing to the last element in the sequence, or to null 
     * if there are no elements in the sequence
     * 
	 * @pre void
	 * @post an iterator has been returned pointing to the element at 
	 * the end of the sequence or to null if size() == 0.
	 * @return an IMiniSequenceIterator<E> iterator to the sequence 
     * pointing to the last element in the sequence
     */
    IMiniSequenceIterator<E> end();  
    
    /**
     * Returns an IMiniSequenceIterator<E> iterator to the sequence 
     * pointing to the element in the sequence with the specified
	 * index value.
     * 
	 * @throws an IndexOutOfBoundsException if index < 0 or index >= size()
	 * @pre 0 <= index < size()
	 * @post an iterator has been returned pointing to the element at 
	 * the specified index in the sequence.
	 * @return an IMiniSequenceIterator<E> iterator to the sequence 
     * pointing to the element at the specified index.
     */
    IMiniSequenceIterator<E> seqIterator(int index);
    
    /**
     * Changes the access mode of the sequence to reflect changes 
     * in the underlying implementation.
	 * @pre void
	 * @post getAccessMode() == accessMode
	 * @param accessMode the accessMode describing the current access mode of
	 * the sequence.  
     */
    void resetAccessMode(AccessMode accessMode);

}
